/*!
 * © 2022 David Vines
 *
 * Creative Commons License
 *
 * This work is licensed under a Creative Commons Attribution-NonCommercial-ShareAlike 4.0 International License.
 */
/* globals ROT, Map, Player, Util, PlayerOrders, console */
/* exported stylenames */

const InitialState = { version: "8", turn: 1 }; /* Initial Game State - version indicates which version of save data we create and can read */
const stylenames = ['player', 'alkari', 'auron', 'caffe', 'clanger', 'drahmini', 'gobb', 'iyanden', 'jerandi', 'kalli', 'kelle',
	'pican', 'regul', 'saggitt', 'strata', 'fleet', 'worati'];
const techs = [0, 8, 9, 6, 6, 5, 3, 1, 8, 8, 3, 9, 4, 5, 3, 2, 0];
var Game = {
	STYLENAMES: stylenames,
	FULLNAMES: {'player': "You",
				'alkari': "the Alkari",
				'auron':"the Aurons",
				'caffe':"the Caffenians",
				'clanger':"the Clangers",
				'drahmini':"the Drahmini",
				'gobb':"the Gobbannanean Principalities",
				'iyanden':"the Chroniomancy of Iyanden",
				'jerandi':"the Jerandi People's Republic'",
				'kalli':"the Kallikanzaroi",
				'kelle':"the Kellesonians",
				'pican':"the Picans",
				'regul':"the Regul",
				'saggitt':"the Saggittarians",
				'strata':"Strata Inc.",
				'fleet':"the Fleet",
				'worati':"the Worati"
				},
	State: { version: undefined, baseMap: undefined, playerMap: undefined, turn: undefined, playerProduction: undefined },
	Players: [],
	History: { baseMap: {} },
	_showMap: function(map) {
		map.forEachHex(function(hex) {
			if (hex.lastScanned) {
				const hexelement = document.getElementById(hex.id);
				const planetElement = hexelement.childNodes[3];
				planetElement.classList.remove("emptyspace");
				planetElement.classList.remove("planet");
				planetElement.classList.add(hex.planet ? "planet" : "emptyspace");
				hexelement.classList.remove(...stylenames);
				if (hex.owner && hex.owner !== "hidden") {
					hexelement.classList.add(hex.owner);
				}
				let text = "";
				if (hex.level) {
					text = "Class "+hex.level+" planet\n";
				}
				if (hex.owner) {
					text += "Owned by "+Game.FULLNAMES[hex.owner]+"\n";
				}
				hexelement.title = text + "Last scanned on Turn " + hex.lastScanned;
			}
		});
	},
	_generateNewGame: function() {
		Game.State = Object.assign({}, InitialState);
		Game.State.baseMap = new Map();
		Game.State.playerMap = new Map(Game.State.baseMap);
		Game.State.playerMap.forEachHex(function(hex) {
			if (hex.owner !== Game.STYLENAMES[0]) {
				Game.State.playerMap.setOwner(hex.id,"hidden"); // i.e. don't reveal the real owner
			} else {
				Game.State.playerMap.setLastScanned(hex.id,Game.State.turn);
			}
		});
		Game.History = { baseMap: {}, version: Game.State.version };
		Game.State.baseMap.forEachHex(function(hex) {
			if (hex.planet) Game.History.baseMap[hex.id] = true;
		}); /* basemap shows where the planets are */

		// Now create the players
		Game._createPlayers(document.getElementById("favouredTech").selectedIndex);

		document.getElementById("welcome").classList.replace("visible", "hidden");
		document.getElementById("summary").classList.replace("hidden", "visible");
		PlayerOrders.setPlayer(Game.Players[0]);
		PlayerOrders.setPlayerMap(Game.State.playerMap);
		Game.State.turn = 0;
		Game._newTurn();
	},
	_createPlayers: function(playerTechIndex) {
		Game.Players = [new Player(0,playerTechIndex)];
		for(let i=1; i< Game.STYLENAMES.length; i++) {
			Game.Players.push(new Player(i,techs[i]));
		}
	},
	_newTurn: function() {
		Game.State.turn++;
		Game._updateHistory();
		Game._saveGameState();
		Game._startTurn();
	},
	_startTurn: function() {
		Game._showMap(Game.State.playerMap);
		Game._clearDisplayTables();
		Game._showResearchReports();
		Game._showColonyReports();
		PlayerOrders._prepare();
		Util.replaceText("turnnumber", Game.State.turn);
	},
	_clearDisplayTables: function() {
	// Clear existing rows after the templates
		for(let templaterow of document.querySelectorAll(".template")) {
			while (templaterow.nextSibling) {
				templaterow.parentNode.removeChild(templaterow.nextSibling);
			}
		}
	},
	// Side effect: Also compute total production - this may (probably will) need to be refactored at some point
	_showColonyReports: function() {
		// Find the colony listing table
		let colonies = document.getElementById("colonies");
		let production = 0;
		Game.State.playerMap.forEachHex(function(hex) {
			if (hex.owner == Game.STYLENAMES[0]) {
				const newrow = colonies.insertRow();

				const rowname = document.createElement("td");
				rowname.append(hex.id);

				const report = document.createElement("td");
				report.append("Nothing to report");

				const marines = document.createElement("td");
				marines.append(hex.marines);

				const productionCell = document.createElement("td");
				productionCell.append(hex.level * 20);

				newrow.append(rowname,report,marines,productionCell);
				production += hex.level * 20;
			}
		});
		Game.Players[0].production = production;
	},
	_showResearchReports: function() {
		Game.Players[0].forEachTech( (tech, research) => {
			Util.replaceText(tech.domPrefix+"Points",research);
			let level = tech.level(research);
			Util.replaceText(tech.domPrefix+"Level",level);
			while(level > 1) {
				document.getElementById(tech.domPrefix+level).classList.remove("hidden");
				level--;
			}
		});
	},
	_updateHistory: function() {
		let sheet = {};
		Game.State.baseMap.forEachHex(function(hex) {
			if (hex.owner) sheet[hex.id] = hex.owner; /* Each turn has a map showing who owns the hex */
		});
		Game.History[Game.State.turn] = sheet;
		localStorage.domainOfTheAncientsHistory = JSON.stringify(Game.History);
	},
	init: function() {
		document.getElementById("generate").onclick = Util.captureExceptionFunction(Game._generateNewGame);
		if (localStorage.domainOfTheAncients && localStorage.domainOfTheAncientsVersion && localStorage.domainOfTheAncientsSaveDate) {
			Game._showLoadButton();
			let v;
			try {
				v = JSON.parse(localStorage.domainOfTheAncientsVersion);
			} catch(_e) {
				v = null; // Clearly no version data if we can't parse it!
			}
				if (v && v.game && (v.game == InitialState.version)) {
					document.getElementById("resume").disabled = false;
					document.getElementById("wrongVersion").classList.replace("visible","hidden");
					document.getElementById("resume").onclick = Util.captureExceptionFunction(Game._load);
				} else {
					document.getElementById("resume").disabled = true;
					if (v && v.plugin) {
						Util.replaceText("incompatibleVersionNumber",v.plugin);
					}
					document.getElementById("wrongVersion").classList.replace("hidden","visible");
				}
		} else {
			document.getElementById("resume").disabled = true;
		}
		if (localStorage.domainOfTheAncientsHistory) {
			// Nothing yet!
		} else {
			document.getElementById("replay").disabled = true;
		}
		// Look for the twisties and set them up
		for(let button of document.querySelectorAll("h4 button")) {
			// Look for the child svg's 'we are interested in
			let svg1 = null, svg2 = null, followingdiv = null;
			for(let child of button.childNodes) {
				if (child.nodeName == "svg") {
					if (svg1 === null) {
						svg1 = child;
					} else {
						svg2 = child;
					}
				}
			}
			// And then the uncle's' (i.e. parent's sibling) div
			let sibling = button.parentNode.nextSibling;
			while(followingdiv === null && sibling) {
				if (sibling.nodeName === "DIV") {
					followingdiv = sibling;
				} else {
					sibling = sibling.nextSibling;
				}
			}

			if (svg1 && svg2 && followingdiv) {
				button.onclick = Util.captureExceptionFunction(Util.toggleFields,[svg1,svg2,followingdiv]);
			}
		}

		PlayerOrders.init();

		// And now the process orders button
		document.getElementById("nextTurn").onclick = Util.captureExceptionFunction(Game._nextTurn);
	},
	_nextTurn: function() {
		let summary = document.getElementById("summary");
		let proc = document.getElementById("processOrder");
		summary.classList.add("hidden");
		proc.classList.remove("hidden");

		let playerOrders = PlayerOrders.getOrders();
		for(let order of playerOrders) {
			let update = order.apply(Game.Players[0],Game.State.baseMap);
			if (update) {
				console.log(update);
			}
		}
		debugger; // jshint ignore:line
		proc.classList.add("hidden");
		summary.classList.remove("hidden");
		Game._newTurn();
	},
	_showLoadButton: function() {
		const saved = new Date(localStorage.domainOfTheAncientsSaveDate);
		const nextUpdate = Util.next_time_diff(saved);
		Util.replaceText("savegamedate",Util.time_diff(saved));
		if (nextUpdate) {
			window.setTimeout(Util.captureExceptionFunction(Game._showLoadButton, nextUpdate*1000));
		}
	},
	_saveGameState: function() {
		let data = {};
		data.ROTRNGstate = JSON.stringify(ROT.RNG.getState());
		data._turn = Game.State.turn;
		data._baseMap = Game.State.baseMap.stringify();
		data._playerMap = Game.State.playerMap.stringify();
		data._playerTech = Game.Players[0].techIndex;
		data._players = [];
		for(let p of Game.Players) {
			data._players.push(p.stringify());
		}
		localStorage.domainOfTheAncientsSaveDate = (new Date()).toISOString();
		let version = {game: Game.State.version, plugin:"3.170.0"};
		localStorage.domainOfTheAncientsVersion = JSON.stringify(version);
		localStorage.domainOfTheAncients = JSON.stringify(data);
	},
	_load: function() {
		const data = JSON.parse(localStorage.domainOfTheAncients);
		ROT.RNG.setState(JSON.parse(data.ROTRNGstate));
		Game.State.turn = data._turn;
		Game.State.baseMap = new Map(data._baseMap);
		Game.State.playerMap = new Map(data._playerMap,Game.State.baseMap);
		Game._createPlayers(data._playerTech);
		for(let i=0; i<Game.Players.length; i++) {
			Game.Players[i].setFromJSON(data._players[i]);
		}
		PlayerOrders.setPlayer(Game.Players[0]);
		PlayerOrders.setPlayerMap(Game.State.playerMap);

		const v = JSON.parse(localStorage.domainOfTheAncientsVersion);
		Game.State.version = v.game;

		document.getElementById("welcome").classList.replace("visible", "hidden");
		document.getElementById("summary").classList.replace("hidden", "visible");
		Game._startTurn();
	},
};

window.onload = function() {
	Util.captureException(Game.init);
};
