/*!
 * © 2022 David Vines
 *
 * Creative Commons License
 *
 * This work is licensed under a Creative Commons Attribution-NonCommercial-ShareAlike 4.0 International License.
 */
/* globals ROT, Game */
/* exported Map */
class Map {
	constructor(m,baseMap) {
		if (m instanceof Map) {
			const us = this;
			this._baseMap = m;
			this.forEachHex(function(hex) {
				if (m[hex.id]) {
					us[hex.id] = {}; // Ensure there is an object at this hex
					if (m[hex.id].lastScanned) {
						us[hex.id].lastScanned = m[hex.id].lastScanned;
					}
				}
			});
			this._baseMap = m;
		} else if (typeof m === 'string') {
			// We assume that the String came from a stringify method call
			if (baseMap) {
				this._baseMap = baseMap; // We were also given a basemap, so we can use that as the basemap
			} else {
				this._baseMap = this; // This is the base map
			}
			// JSONify the string and then use that to build our map
			const json = JSON.parse(m);
			const us = this;
			this.forEachHex(function(hex) {
				if (json[hex.id]) {
					us[hex.id] = json[hex.id];
				}
			});
		} else {
			// New base map
			this._baseMap = this; // We are the base map
			let possibles = {};
			this.forEachHex(function(hex){
				const col = hex.id[0].charCodeAt() - 'A'.charCodeAt() + 1;
				const row = parseInt(hex.id.slice(1));
				const distance = Math.sqrt(Math.abs(row-10.5)*Math.abs(row-10.5) + Math.abs(col-8.5)*Math.abs(col-8.5) );
				possibles[hex.id] = 15-distance;
			});
			for(let i=0; i<100; i++) {
				const choice = ROT.RNG.getWeightedValue(possibles);
				this._baseMap[choice] = {}; // Ensure there is an object at this hex
				this._baseMap[choice].planet = true;
				if (i < Game.STYLENAMES.length) {
					this._baseMap[choice].owner = Game.STYLENAMES[i];
					this._baseMap[choice].level = 20;
					this._baseMap[choice].marines = 0;
					this._baseMap[choice].sensors = 0;
				}
				delete possibles[choice];
			}
		}
	}
	hex(name) {
		return Object.assign({planet: false, id: name}, this._baseMap[name], this[name]);
	}
	improveColony(colony) {
		if (this !== this._baseMap) {
			this._baseMap.improveColony(colony);
		} else {
			this[colony].level++;
		}
	}
	addMarines(colony,companies) {
		if (this !== this._baseMap) {
			this._baseMap.addMarines(colony,companies);
		} else {
			this[colony].marines += parseInt(companies);
		}
	}
	addSensorBuilding(colony) {
		if (this !== this._baseMap) {
			this._baseMap.addSensorBuilding(colony);
		} else {
			this[colony].sensors++;
		}
	}
	forEachHex(f) {
		const self = this;
		for(let row=1; row<21; row++) {
			for(let column of "ABCDEFGHIJKLMNOP") {
				const name = column+row;
				f(self.hex(name));
			}
		}
	}
	stringify() {
		let data = {};
		for(let row=1; row<21; row++) {
			for(let column of "ABCDEFGHIJKLMNOP") {
				const name = column+row;
				if (this[name]) data[name] = this[name];
			}
		}
		return JSON.stringify(data);
	}
	setLastScanned(hexname,turn) {
		if (this[hexname]) this[hexname].lastScanned = turn;
	}
	setOwner(hexname,o) {
		if (this[hexname]) this[hexname].owner = o;
	}
}