/*!
 * © 2022 David Vines
 *
 * Creative Commons License
 *
 * This work is licensed under a Creative Commons Attribution-NonCommercial-ShareAlike 4.0 International License.
 */
/* exported ResearchOrder, CarryForwardOrder, ImproveColonyOrder, AddMarinesOrder, AddSensorBuildingOrder */
class ResearchOrder {
	constructor(tech,amount) {
		this._tech = tech;
		this._amount = amount;
	}
	toString() {
		return "Research "+this._amount+" in "+this._tech.option;
	}
	apply(player,_map) {
		if (player.addResearch(this._tech,this._amount))
		{
			return this._tech.option+" has now reached level "+player.researchLevel(this._tech);
		}
		return null;
	}
}

class CarryForwardOrder {
	constructor(amount) {
		this._amount = amount;
	}
	toString() {
		return "carry forward "+this._amount+" PPs";
	}
	apply(player,_map) {
		player.carriedProduction = this._amount;
		return null;
	}
}

class ImproveColonyOrder {
	constructor(colony) {
		this._colony = colony;
	}
	get colony() {
		return this._colony;
	}
	toString() {
		return "Improve colony at "+this._colony;
	}
	apply(_player,map) {
		map.improveColony(this._colony);
		return null;
	}
}

class AddMarinesOrder {
	constructor(colony,companies) {
		this._colony = colony;
		this._companies = companies;
	}
	get colony() {
		return this._colony;
	}
	get companies() {
		return this._companies;
	}
	toString() {
		return "Add "+this._companies+" marine companies to the  colony at "+this._colony;
	}
	apply(_player,map) {
		map.addMarines(this._colony,this._companies);
		return null;
	}
}

class AddSensorBuildingOrder {
	constructor(colony) {
		this._colony = colony;
	}
	get colony() {
		return this._colony;
	}
	toString() {
		return "Add a sensor building to the  colony at "+this._colony;
	}
	apply(_player,map) {
		map.addSensorBuilding(this._colony);
		return null;
	}
}