/*!
 * © 2022 David Vines
 *
 * Creative Commons License
 *
 * This work is licensed under a Creative Commons Attribution-NonCommercial-ShareAlike 4.0 International License.
 */
/* globals Util, ResearchOrder, CarryForwardOrder, ImproveColonyOrder, AddMarinesOrder, AddSensorBuildingOrder, ALLTECHS */
/* exported PlayerOrders */
var PlayerOrders = {
	_purchaseOrderChoice: 0,
	_player: null,
	_map: null,
	_leftOver: 0,
	_improvedColonies: {},
	_coloniesSelectable: [],
	_orders: [],
	init: function() {
		// Set up handler for the purchase option select element
		document.getElementById("purchase-options").onchange = Util.captureExceptionEventHandler(PlayerOrders._handlePurchaseOptionEvent);

		// Set up the Reseach handlers
		document.getElementById("research-amount-range").oninput  = Util.captureExceptionFunction(PlayerOrders._handleResearchRangeEvent);
		document.getElementById("research-amount-number").oninput = Util.captureExceptionFunction(PlayerOrders._handleResearchNumberEvent);
		document.getElementById("research-topic").onchange = Util.captureExceptionFunction(PlayerOrders._handleResearchNumberEvent);

		// Set up the colony improver select handler
		document.getElementById("improve-colony-select-list").onchange = Util.captureExceptionFunction(PlayerOrders._handleImproveColonyEvent);

		// Set up the marine handlers
		document.getElementById("marines-amount-range").oninput  = Util.captureExceptionFunction(PlayerOrders._handleMarinesRangeEvent);
		document.getElementById("marines-amount-number").oninput = Util.captureExceptionFunction(PlayerOrders._handleMarinesNumberEvent);

		// Set up handler for the "do it" button on the spend points page
		document.getElementById("purchase-option-doit").onclick = Util.captureExceptionFunction(PlayerOrders._handlePurchaseDoit);
	},
	setPlayer: function(player) {
		// Save the reference to the player
		PlayerOrders._player = player;
	},
	setPlayerMap: function(map) {
		PlayerOrders._map = map;
	},
	_showAvailableExpenditureOptions: function() {
		PlayerOrders._showAvailableOptions();
		PlayerOrders._showAvailableColonyOptions();
	},
	_prepare: function() {
		// The chooser
		const chooser = document.getElementById("purchase-options");
		chooser.selectedIndex = 0;
		for(let suboption of document.querySelectorAll(".purchase-option")) {
			let id = suboption.id;
			if (id.slice(-2) == "-0") {
				suboption.classList.remove("hidden");
			} else {
				suboption.classList.add("hidden");
			}
		}
		PlayerOrders._purchaseOrderChoice = 0;

		// Reset the bag of colonies being improved
		PlayerOrders._improvedColonies = {};

		// We start with no orders
		PlayerOrders._orders = [];

		// Reset the totals
		PlayerOrders._updateCost(0);
		PlayerOrders._recomputeFields(document.getElementById("purchase-table"));

		Util.replaceText("savedPPs",PlayerOrders._player.carriedProduction);
		Util.replaceText("productionThisTurn",PlayerOrders._player.production);
		Util.replaceText("availablePPs",PlayerOrders._leftOver);
	},
	_showAvailableOptions: function() {
		document.getElementById("research-amount-range").value = 0;
		document.getElementById("research-amount-number").value = 0;
		document.getElementById("research-topic").selectedIndex = 0;
		document.getElementById("research-amount-range").max = PlayerOrders._leftOver*2;
		document.getElementById("research-amount-range").value = 0;
		document.getElementById("research-amount-number").max = PlayerOrders._leftOver*2;
		document.getElementById("research-amount-number").value = 0;

		const techs = document.getElementById("research-topic");
		let i = 0;
		for(let option of techs.options) {
			// Remove any trailing paraparenthezied phrase (and the space that introduces it)
			option.text = option.text.replace(/ \(.*\)/,"");
			if (i === PlayerOrders._player.techIndex) {
				option.text = option.text + " (favoured tech)";
			}
			i++;
		}

		document.getElementById("marines-amount-range").value = 0;
		document.getElementById("marines-amount-number").value = 0;
		document.getElementById("marines-amount-range").max = Math.floor(PlayerOrders._leftOver/10);
		document.getElementById("marines-amount-range").value = 0;
		document.getElementById("marines-amount-number").max = Math.floor(PlayerOrders._leftOver/10);
		document.getElementById("marines-amount-number").value = 0;
	},
	_showAvailableColonyOptions: function() {
		PlayerOrders._coloniesSelectable.length = 0;
		const colonyList = document.getElementById("improve-colony-select-list");
		const sensorList = document.getElementById("sensor-colony-select-list");
		const marineList = document.getElementById("marines-colony-select-list");
		let firstenabled = -1;
		let row = 0;
		Util.emptyList(colonyList);
		PlayerOrders._map.forEachHex(function(hex) {
			if (hex.owner == PlayerOrders._player.stylename) {
				const colonyOption = document.createElement("option");
				const sensorOption = document.createElement("option");
				const marineOption = document.createElement("option");
				colonyOption.append(hex.id);
				sensorOption.append(hex.id);
				marineOption.append(hex.id);
				if (PlayerOrders._improvedColonies[hex.id]) {
					colonyOption.disabled = true;
				} else if (firstenabled < 0){
					firstenabled = row;
				}
				PlayerOrders._coloniesSelectable.push(hex.id);
				colonyList.append(colonyOption);
				sensorList.append(sensorOption);
				marineList.append(marineOption);
				row++;
			}
		});
		colonyList.disabled = (firstenabled === -1);
		colonyList.selectedIndex = firstenabled;
		if (PlayerOrders._purchaseOrderChoice === 1) {
			PlayerOrders._handleImproveColonyEvent();
		}
		if (PlayerOrders._purchaseOrderChoice === 2) {
			PlayerOrders._handleSensorColonyEvent();
		}
		if (PlayerOrders._purchaseOrderChoice === 4) {
			PlayerOrders._handleMarinesColonyEvent();
		}
	},
	_updateCost(cost) {
		Util.replaceText("purchase-option-cost",cost);
		if (cost) {
			document.getElementById("purchase-option-doit").disabled = false;
		} else {
			document.getElementById("purchase-option-doit").disabled = true;
		}
	},
	_handlePurchaseOptionEvent: function() {
		document.getElementById("purchase-option-"+PlayerOrders._purchaseOrderChoice).classList.add("hidden");
		PlayerOrders._purchaseOrderChoice = document.getElementById("purchase-options").selectedIndex;
		document.getElementById("purchase-option-"+PlayerOrders._purchaseOrderChoice).classList.remove("hidden");
		let cost = 0;
		if (PlayerOrders._purchaseOrderChoice === 0) {
			// Improve a tech
			cost = document.getElementById("research-amount-number").value;
			if (document.getElementById("research-topic").selectedIndex == PlayerOrders._player.techIndex) {
				cost = Math.ceil(cost/2);
			}
			PlayerOrders._updateCost(cost);
		} else if (PlayerOrders._purchaseOrderChoice === 1) {
			// Improve a colony
			PlayerOrders._handleImproveColonyEvent(); // push a selection of the current choice to get the cost updated
		} else if (PlayerOrders._purchaseOrderChoice === 2) {
			PlayerOrders._updateCost(150);
		} else if (PlayerOrders._purchaseOrderChoice === 4) {
			PlayerOrders._updateCost(document.getElementById("marines-amount-number").value*10);
		}
	},
	_handleResearchRangeEvent: function() {
		PlayerOrders._handleResearchUpdate(document.getElementById("research-amount-range").value);
	},
	_handleResearchNumberEvent: function() {
		PlayerOrders._handleResearchUpdate(document.getElementById("research-amount-number").value);
	},
	_handleResearchUpdate: function(newValue) {
		document.getElementById("research-amount-range").value = newValue;
		document.getElementById("research-amount-number").value = newValue;
		let cost = newValue;
		if (document.getElementById("research-topic").selectedIndex == PlayerOrders._player.techIndex) {
			cost = Math.ceil(cost/2);
		}
		PlayerOrders._updateCost(cost);
	},
	_handleMarinesRangeEvent: function() {
		PlayerOrders._handleMarinesColonyEvent(document.getElementById("marines-amount-range").value);
	},
	_handleMarinesNumberEvent: function() {
		PlayerOrders._handleMarinesColonyEvent(document.getElementById("marines-amount-number").value);
	},
	_handleSensorColonyEvent: function() {
		PlayerOrders._updateCost(150);
	},
	_handleMarinesColonyEvent: function(newValue) {
		document.getElementById("marines-amount-range").value = newValue;
		document.getElementById("marines-amount-number").value = newValue;
		PlayerOrders._updateCost(newValue*10);
	},
	_handleImproveColonyEvent: function() {
		const index = document.getElementById("improve-colony-select-list").selectedIndex;
		let cost = 0;
		if (index !== -1) {
			cost = PlayerOrders._map.hex( PlayerOrders._coloniesSelectable[index]).level * 20;
		}
		PlayerOrders._updateCost(cost);
	},
	_handlePurchaseDoit: function() {
		const cost = document.getElementById("purchase-option-cost").innerText;
		const table = document.getElementById("purchase-table");
		const newrow = table.insertRow();
		let purchase = "undefined";
		let order = null;
		if (PlayerOrders._purchaseOrderChoice === 0) {
			// Purchase a technology
			const topic = ALLTECHS[document.getElementById("research-topic").selectedIndex];
			const amount = document.getElementById("research-amount-number").value;
			purchase = `Research ${amount} in ${topic.option}`;
			order = new ResearchOrder(topic,amount);
		} else if (PlayerOrders._purchaseOrderChoice === 1) {
			// Purchase colony upgrade
			const index = document.getElementById("improve-colony-select-list").selectedIndex;
			const colony = PlayerOrders._coloniesSelectable[index];
			purchase = `Improve ${colony}`;
			order = new ImproveColonyOrder(colony);
		} else if (PlayerOrders._purchaseOrderChoice === 2) {
			// Purchase colony upgrade
			const colony = document.getElementById("sensor-colony-select-list").selectedOptions[0].text;
			purchase = `Buy a new sensor building at ${colony}`;
			order = new AddSensorBuildingOrder(colony);
		} else if (PlayerOrders._purchaseOrderChoice === 4) {
			const colony = document.getElementById("marines-colony-select-list").selectedOptions[0].text;
			const amount = document.getElementById("marines-amount-number").value;
			if (amount == 1) {
				purchase = `Add a marine company to ${colony}`;
			} else {
				purchase = `Add ${amount} marine companies to ${colony}`;
			}
			order = new AddMarinesOrder(colony,amount);
		}

		// Add the purchase cell
		PlayerOrders._orders.push(order);
		const cell = document.createElement("td");
		cell.append(purchase);
		newrow.append(cell);

		// Add cost cell
		newrow.dataset.cost = cost;
		const costcell = document.createElement("td");
		costcell.append(cost);

		// Add a cancel button
		const cancel = document.createElement("button");
		cancel.append("Cancel");
		cancel.onclick = Util.captureExceptionFunction(PlayerOrders._cancelPurchaseRow,newrow);

		newrow.append(costcell,cancel);
		PlayerOrders._recomputeFields(table);
		PlayerOrders._showAvailableExpenditureOptions();
	},
	_cancelPurchaseRow: function(row) {
		let table = row.parentNode;
		let index = row.rowIndex;
		if (table.nodeName == "TBODY") { table = table.parentNode; } // Skip the tbody if present
		table.deleteRow(index);
		PlayerOrders._orders.splice(index-3,1); // -3 to account for the header, the purchase selection and the template row (which is used to clear the table for a new turn)
		PlayerOrders._recomputeFields(table);
	},
	_recomputeFields: function(table) {
		let total = 0;
		for(let row of table.rows) {
			if (row.dataset && row.dataset.cost !== undefined) {
				total += parseInt(row.dataset.cost);
			}
		}
		Util.replaceText("totalExpenditure",total);
		if (total > PlayerOrders._player.production+PlayerOrders._player.carriedProduction) {
			document.getElementById("tooMuchExpenditure").classList.replace("hidden","visible");
			document.getElementById("nextTurn").disabled = true;
		} else {
			document.getElementById("tooMuchExpenditure").classList.replace("visible","hidden");
			document.getElementById("nextTurn").disabled = false;
		}
		PlayerOrders._leftOver = PlayerOrders._player.production+PlayerOrders._player.carriedProduction-total;
		Util.replaceText("carriedForwardPPs",PlayerOrders._leftOver);

		// Record the colonies being improved so that they can't be selected again
		PlayerOrders._improvedColonies = {};
		for(let order of PlayerOrders._orders) {
			if (order instanceof ImproveColonyOrder) {
				PlayerOrders._improvedColonies[order.colony] = true;
			}
		}

		// Now we know the left over funds reset the expenditure options
		PlayerOrders._showAvailableExpenditureOptions();
	},
	getOrders: function() {
		PlayerOrders._orders.push(new CarryForwardOrder(PlayerOrders._leftOver));
		return PlayerOrders._orders;
	}
};
