/*!
 * © 2022 David Vines
 *
 * Creative Commons License
 *
 * This work is licensed under a Creative Commons Attribution-NonCommercial-ShareAlike 4.0 International License.
 */
/* exported Util */
var Util= {
		captureException: function(f, arg) {
		try {
			f(arg);
		} catch (ex) {
			const exdiv = document.getElementById("exception");
			exdiv.className = exdiv.className.replace(/exhidden/, "exvis");
			const element = document.getElementById("exception.text");
			let text;
			if (ex instanceof Error) {
				text = ex.name + " : " + ex.message;
			} else {
				text = ex.toString();
			}
			text = text + '\n' + (ex.stack || ex.stacktrace || '');

			// Remove the chunk in the stack trace between @ and /js/domainOfTheAncients/
			text = text.replace(/@.*\/js\/domainOfTheAncients\//g, " in ");
			// And version field too
			text = text.replace(/\?ver=[0-9a-fA-F]*:/g, ":");

			const newText = document.createTextNode(text);
			if (element.hasChildNodes()) {
				element.replaceChild(newText, element.firstChild);
			} else {
				element.appendChild(newText);
			}
			debugger; // jshint ignore:line
		}
	},
	captureExceptionFunction: function(f, arg) {
		return function() {
			Util.captureException(f, arg);
		};
	},
	captureExceptionEventHandler: function(f) {
		return function(e) {
			Util.captureException(f, e);
		};
	},
	emptyList: function(id) {
		const element = (typeof id === "string" ? document.getElementById(id) : id);
		while (element.firstChild) {
			element.removeChild(element.firstChild);
		}
		return element;
	},
	replaceText: function(id, text) {
		let node = (typeof id === "string" ? document.getElementById(id) : id);
		node.replaceChild(document.createTextNode(text), node.firstChild);
	},
	toggleFields(fields) {
		for(let field of fields) {
			field.classList.toggle("hidden");
		}
	},
	time_diff: function(from) {
		const diff = Math.floor(Math.abs(new Date().getTime() - from.getTime()) / 1000); // In seconds
		if (diff < 60) return "just now";
		if (diff < 120) return "a minute ago";
		if (diff < 3600) return Math.floor(diff / 60) + " minutes ago";
		if (diff < 7200) return "an hour ago";
		if (diff < 86400) return Math.floor(diff / 3600) + " hours ago";
		if (diff < 86400 * 2) return "yesterday";
		if (diff < 86400 * 6) return Intl.DateTimeFormat(undefined, { weekday: "long" }).format(from);
		if (new Date().getFullYear() === from.getFullYear()) return Intl.DateTimeFormat(undefined, { day: "numeric", month: "long" }).format(from);
		return Intl.DateTimeFormat(undefined, { day: "numeric", month: "short", year: "numeric" }).format(from);
	},
	next_time_diff: function(from) {
		const diff = Math.floor(Math.abs(new Date().getTime() - from.getTime()) / 1000); // In seconds
		if (diff < 3660) return 60;
		if (diff < 86400) return 3600;
		return null;
	}
};
